/* Name: main.c
 * Based on project: hid-data, example how to use HID for data transfer
 * (Uses modified descriptor and usbFunctionSetup from it.)
 * Original author: Christian Starkjohann
 * Arduino modifications by: Philip J. Lindsay
 * Creation Date: 2008-04-11
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 692 2008-11-07 15:07:40Z cs $
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.
*/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/eeprom.h>

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include "oddebug.h"        /* This is also an example for using debug macros */

#include "UsbStreamMultibyte.h"

UsbStreamMultibyteDevice::UsbStreamMultibyteDevice(){}

void UsbStreamMultibyteDevice::begin()
{
    // disable timer 0 overflow interrupt (used for millis)
    TIMSK0&=!(1<<TOIE0);

    cli();

    usbInit();

    usbDeviceDisconnect();
    uchar   i;
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        _delay_ms(1);
    }
    usbDeviceConnect();

    sei();
}

void UsbStreamMultibyteDevice::end()
{
    // de enumerate
    usbDeviceDisconnect();
    // enable timer 0 overflow interrupt (used for millis)
    TIMSK0 |= (1<<TOIE0);
}

static uchar tx_buf[32];
static uchar rx_buf[32];
static bool new_rx_data_avail;
    
void UsbStreamMultibyteDevice::refresh()
{
  usbPoll();
}

bool UsbStreamMultibyteDevice::available()
{
    return new_rx_data_avail;
}

void UsbStreamMultibyteDevice::write(char* buf, uchar len)
{
    memcpy(tx_buf, buf, len);
}

void UsbStreamMultibyteDevice::read(char* buf, uchar len)
{
    memcpy(buf, rx_buf, len);
    new_rx_data_avail = false;
}


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

#ifdef __cplusplus
extern "C"{
#endif 
PROGMEM char usbHidReportDescriptor[22] = {    /* USB report descriptor */
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x20,                    //   REPORT_COUNT (32)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};
/* Since we define only one feature report, we don't use report-IDs (which
 * would be the first byte of the report). The entire report consists of 32
 * opaque data bytes.
 */

/* ------------------------------------------------------------------------- */

// Status of the current data transfer:
static uchar currentAddress;
static uchar bytesRemaining;

// Called when host requests a chunk of data from the device
uchar usbFunctionRead(uchar *data, uchar len)
{
    if (len > bytesRemaining)
        len = bytesRemaining;
    
    // Actually do the data copy from device to host
    memcpy(data, tx_buf + currentAddress, len);

    currentAddress += len;
    bytesRemaining -= len;
    return len;
}

// called when the host sends a chunk of data to the device
uchar usbFunctionWrite(uchar *data, uchar len)
{
    if (bytesRemaining == 0)
        return 1; // end of transfer
    if (len > bytesRemaining)
        len = bytesRemaining;

    // Actually do the data copy from host to device
    memcpy(rx_buf + currentAddress, data, len);

    currentAddress += len;
    bytesRemaining -= len;
    if (bytesRemaining == 0)
    {
        new_rx_data_avail = true;
        return 1; // return 1 if this was the last chunk host to device
    }
    return 0;
}

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
  usbRequest_t    *rq = (usbRequest_t*)((void *)data);

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) // HID class request
    {
        if(rq->bRequest == USBRQ_HID_GET_REPORT) // wValue: ReportType (highbyte), ReportID (lowbyte)
        {
            // since we have only one report type, we can ignore the report-ID
            bytesRemaining = 32;
            currentAddress = 0;
            return USB_NO_MSG; // use usbFunctionRead() to receive data from the host
        }
        else if(rq->bRequest == USBRQ_HID_SET_REPORT)
        {
            // since we have only one report type, we can ignore the report-ID
            bytesRemaining = 32;
            currentAddress = 0;
            return USB_NO_MSG; // use usbFunctionWrite() to receive data from the host
        }
    }else{
        // ignore vendor type requests, we don't use any
    }
    return 0;
}
#ifdef __cplusplus
} // extern "C"
#endif

UsbStreamMultibyteDevice UsbStreamMultibyte = UsbStreamMultibyteDevice();


