/*Nerdle Calibration Sketch
7/21/2010

This sketch shows the value read by each of the four buttons.
It's quickly updated and responds to input.

You can use this sketch to adjust the threshold for your buttons
to change what Nerdle considers as "pushed."

Press each of the four buttons, and take note of a value
that is exceeded when you push each button, but doesn't happen
without touch input.

This is often a number around 200.

Uploading instructions:  
Bootloader mode is used for updating the sketch on the device,
not updating words.

To upload sketches to your Nerdle, connect the  Nerdle to the PC
using the mini-b USB cable.  Then, press the reset button on the
Nerdle.  The screen should go blank, and the device is now in 
"bootloader mode." 

Upload a sketch using the keyboard shortcut Ctrl-U or using the
upload button in the top bar.  The sketch will run on the 
device after it is uploaded.  To upload again, keep the device 
plugged into USB and press reset again.

If you enter bootloader mode and don't want to upload anything,
unplug it from USB, and press reset.  The stored program will
run.

*/
#define UP_THRESHOLD 150
#define DOWN_THRESHOLD 150
#define LEFT_THRESHOLD 150
#define RIGHT_THRESHOLD 150



#include <LiquidCrystal.h>
#include <CapSense.h>



/*Nerdle Calibration
 http://wayneandlayne.com/nerdle

 Adam Wolf & Matthew Beckler
 Wayne and Layne
 */



/* Hardware Definitions */
// LCD
#define LCD_POWER 0
#define LCD_RS 6
#define LCD_E 5
#define LCD_DB4 10
#define LCD_DB5 11
#define LCD_DB6 12
#define LCD_DB7 13

// USB
#define USB_VUSB 3

// Piezo speaker
#define SPEAKER_PIN 9

// Capacitive-touch buttons
#define BUTTON_LF_PIN 16
#define BUTTON_RT_PIN 7
#define BUTTON_UP_PIN 8
#define BUTTON_DN_PIN 15
#define BUTTON_COUNT 5
#define SEND_PIN 17



/* Button setup */
#define BUTTON_UP 0
#define BUTTON_RT 1
#define BUTTON_LF 2
#define BUTTON_DN 3



#define LCD_COLS 16


LiquidCrystal lcd(LCD_RS, LCD_E, LCD_DB4, LCD_DB5, LCD_DB6, LCD_DB7); 


CapSense   cs_up = CapSense(SEND_PIN,BUTTON_UP_PIN); //1M between pins w foil pad attached to second pin
CapSense   cs_right = CapSense(SEND_PIN,BUTTON_RT_PIN); //1M between pins w foil pad attached to second pin
CapSense   cs_left = CapSense(SEND_PIN,BUTTON_LF_PIN); //1M between pins w foil pad attached to second pin
CapSense   cs_down = CapSense(SEND_PIN,BUTTON_DN_PIN); //1M between pins w foil pad attached to second pin


void lcd_setup()
{
  pinMode(LCD_POWER, OUTPUT);
  digitalWrite(LCD_POWER, HIGH);
  delay(50);
  lcd.begin(16, 2);
  delay(50);
  lcd.clear();
  lcd.print("Nerdle 2.02 Self");
  lcd.setCursor(0, 1);
  lcd.print("Test");
  
  
}

void setup()
{
  pinMode(SPEAKER_PIN, OUTPUT);
  tone(SPEAKER_PIN, 4000);
  delay(300);
  noTone(SPEAKER_PIN);
  lcd_setup();
  tone(SPEAKER_PIN, 1000);
  delay(300);
  noTone(SPEAKER_PIN);
  
}

void loop()
{

  static char button_was_pushed[BUTTON_COUNT] = {
    0,0,0,0,0      };

  static char button_state[BUTTON_COUNT] = {
    0, 0, 0, 0, 0     };




  long total0 =  cs_up.capSense(30);
  long total1 =  cs_right.capSense(30);
  long total2 =  cs_left.capSense(30);
  long total3 =  cs_down.capSense(30);

  
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(total0);
  lcd.setCursor(8, 0);
  lcd.print(total1);
  lcd.setCursor(0,1);
  lcd.print(total2);
  lcd.setCursor(8,1);
  lcd.print(total3);

  if (total0 > UP_THRESHOLD)
  {
    if (button_state[0] == 1)
    {
      //it is currently pushed, and last time it was pushed.
      button_was_pushed[0] = 0;
    } 
    else
    {
      //it is now pushed and last time it wasn't
      tone(SPEAKER_PIN, 1000, 10);
      button_was_pushed[0] = 1;
    }

    button_state[0] = 1;
  }
  else
  {
    if (button_state[0] == 1)
    {
      //we weren't pushed, but now we are!
      tone(SPEAKER_PIN, 500, 10);
    }
    button_was_pushed[0] = 0;
    button_state[0] = 0;
  }


  if (total1 > RIGHT_THRESHOLD)
  {
    if (button_state[1] == 1)
    {
      button_was_pushed[1] = 0;
    } 
    else
    {
      tone(SPEAKER_PIN, 1000, 10);
      button_was_pushed[1] = 1;
    }
    button_state[1] = 1;
  }
  else
  {
    if (button_state[1] == 1)
    {
      //we weren't pushed, but now we are!
      tone(SPEAKER_PIN, 500, 10);
    }
    button_was_pushed[1] = 0;
    button_state[1] = 0;
  }

  if (total2 > LEFT_THRESHOLD)
  {

    if (button_state[2] == 1)
    {
      button_was_pushed[2] = 0;
    } 
    else
    {
      tone(SPEAKER_PIN, 1000, 10);

      button_was_pushed[2] = 1;
    }
    button_state[2] = 1;
  }
  else
  {
    if (button_state[2] == 1)
    {
      //we weren't pushed, but now we are!
      tone(SPEAKER_PIN, 500, 10);
    }
    button_was_pushed[2] = 0;
    button_state[2] = 0;
  }

  if (total3 > DOWN_THRESHOLD)
  {
    if (button_state[3] == 1)
    {
      button_was_pushed[3] = 0;
    } 
    else
    {
      tone(SPEAKER_PIN, 1000, 10);
      button_was_pushed[3] = 1;
    }
    button_state[3] = 1;
  }
  else
  {
    if (button_state[3] == 1)
    {
      //we weren't pushed, but now we are!
      tone(SPEAKER_PIN, 500, 10);
    }
    button_was_pushed[3] = 0;
    button_state[3] = 0;
  }



}


